.box {
    position: relative;
    display: inline-block;
    padding: 3px 8px;
    border: 1px solid lightgrey;
    border-radius: 4px;
}

.box.has-inner {
    padding-right: 12px;
}

.box.has_value {
    border-color: grey;
}

.box.selected {
    padding: 1px 6px;
    border: 3px solid #ccc;
}

.box.selected.has_value {
    border-color: #2822ab;
}

.box.has_value.value_none {
    border-style: dashed;
}

.box.has_value.exception_node, .box.has_value.selected.exception_node {
    padding: 1px 6px;
    border: 3px solid red;
}

.box.hovering {
    background-color: #c1fdff;
}

#bottom_panel {
    position: fixed !important;
    bottom: 0 !important;
    margin: 0;
    width: 100%;
}

#box_value, #inspector {
    border: 1px solid lightgrey;
    margin: 0;
    padding: 10px;
    border-radius: 0;
    background: white;
}

#inspector {
    height: 12em;
    overflow: auto;
}

#resize-handle {
    background-color: lightgrey;
    height: 10px;
    width: 100%;
    cursor: ns-resize;
    background-repeat: no-repeat;
    background-position: center;
}

#box_value {
    max-height: 5.7em;
    overflow: hidden;
}

.flex-container {
    display: flex;
    width: 100%;
}

#arrows-holder {
    width: 100px;
    height: 100px;
    display: inline-block;
    position: relative;
}

#code {
    flex-grow: 1;
    display: inline-block;
    margin-bottom: 40em;
    overflow: visible;
}

.code-font, #inspector ul li.jstree-node {
    font-family: Menlo, Monaco, Consolas, "Courier New", monospace;
}

.call-row a {
    display: block;
}

.args-table tr td:first-child {
    max-width: 20em;
}

.loop-label {
    pointer-events: none;
}

.loop-navigator > button, .loop-navigator > .btn-group {
    float: none !important;
}

.jstree-proton .jstree-icon.jstree-themeicon-custom {
    margin-right: 8px;
}

#code span span.stmt_uncovered,
#code span span.stmt_uncovered span[class^="hljs-"] {
    color: #b3b3b3;
    font-weight: normal;
}

.inner-call {
    position: absolute;
    right: 0;
}

.inner-call .glyphicon {
    transform: rotate(45deg);
}

body > .container {
    margin-left: 100px;
}

.jstree-anchor .inspector-value {
    display: inline-block;
    white-space: pre;
    vertical-align: top;
}

#inspector .jstree-anchor {
    height: auto;
}

table.dataframe.table {
    width: auto;
    font-size: 9pt;
    margin: 5px;
    border: 2px solid black;
}

table.dataframe td.numeric {
    text-align: right;
}

.table-striped > tbody > tr:nth-of-type(odd) {
    background-color: #f5f5f5;
}

.table-striped.table-hover > tbody > tr:hover {
    background-color: rgba(66, 165, 245, 0.2);
}

/* Based on https://bootsnipp.com/snippets/949K6 */
.panel.panel-info {
    position: relative;
    /*
    Shrink to fit text:
    https://stackoverflow.com/a/3917059/2482744
    */
    display: inline-block;
}

.panel-leftheading {
    width: 34px;
    text-align: center;
    border-right: 1px solid transparent;
    border-bottom: 1px solid transparent;
    float: left;
    height: 100%;
    position: absolute;
    /*
    Center text vertically:
    https://stackoverflow.com/a/13515693/2482744
     */
    display: flex;
    justify-content: center;
    align-content: center;
    flex-direction: column;
}

.panel-leftheading > .glyphicon {
    font-size: 160%;
}

.panel-rightbody {
    float: left;
    margin-left: 45px;
    padding: 15px;
}

.panel-info > .panel-leftheading {
    color: #31708f;
    background-color: #d9edf7;
    border-color: #bce8f1;
}

.no-bullet {
    list-style-type: none;
}

#new-call {
    display: none;
    position: fixed;
    top: 0;
    right: 0;
    transform-origin: top right;
    transform: scale(1);
}

#new-call.animate {
    animation: size-bounce 0.5s;
}

@keyframes size-bounce {
    0% {
        transform: scale(1);
    }
    50% {
        transform: scale(1.2);
    }
    100% {
        transform: scale(1);
    }
}