{% extends "base.html" %}
{% block head %}
  <link rel="stylesheet" href="{{ url_for('static', filename='css/hljs.min.css') }}">
  <link rel="stylesheet" href="{{ url_for('static', filename='css/proton/style.css') }}">
{% endblock %}
