<div class="flex-container">
  <div id="arrows-holder"></div>
  <pre id="code" class="python">{{ func.html_body | safe }}</pre>
</div>
<div id="bottom_panel">
    <div id="resize-handle"
         class="text-center"
         style="background-image: url({{ url_for('static', filename='img/handle.png') }})">
    </div>
  <div id="inspector"></div>
  <pre id="box_value"></pre>
</div>

<script src="{{ url_for('static', filename='js/libs/highlight.pack.js') }}"></script>
<script>
  var call_success = {{ call.success | tojson | safe }};
  var call_data = {{ call.parsed_data | tojson | safe }};
  var func_data = {{ func.parsed_data | tojson | safe }};
  var static_url = "{{ url_for('static', filename='') }}";
  var call_url = "{{ url_for('call_view', call_id='') }}";
</script>
<script src="{{ url_for('static', filename='js/libs/jstree.js') }}"></script>
<script src="{{ url_for('static', filename='js/libs/underscore.js') }}"></script>
<script src="{{ url_for('static', filename='js/call.js') }}"></script>
